class JavaScriptCompletionsAssistant {
    constructor() {
        this.importableFileExtensions = ["js", "ts", "jsx", "tsx"];
        this.doubleQuotedPathSelector = new ScopeSelector("string.double-quoted.path");
        this.singleQuotedPathSelector = new ScopeSelector("string.single-quoted.path");
    }
    
    provideCompletionItems(editor, context) {
        let selectors = context.selectors;
        
        let matchSelectors = [this.doubleQuotedPathSelector, this.singleQuotedPathSelector];
        
        let matchingSelector = null;
        
        for (let selector of selectors) {
            for (let matchSelector of matchSelectors) {
                if (selector.matches(matchSelector)) {
                    matchingSelector = matchSelector;
                    break;
                }
            }
            if (matchingSelector) {
                break;
            }
        }
        
        if (!matchingSelector) {
            return [];
        }
        
        let line = context.line;
        let substring = null;
        
        // Determine the text before the cursor, but after the quote
        if (matchingSelector == this.doubleQuotedPathSelector) {
            let idx = line.lastIndexOf("\"")
            if (idx != -1) {
                substring = line.substring(idx + 1, line.length);
            }
        }
        else if (matchingSelector == this.singleQuotedPathSelector) {
            let idx = line.lastIndexOf("'")
            if (idx != -1) {
                substring = line.substring(idx + 1, line.length);
            }
        }
        
        if (!substring && substring !== "") {
            return [];
        }
        
        let components = nova.path.split(substring);
        let basePath = null;
        
        if (components[0] == "/") {
            // Absolute in workspace
            basePath = nova.workspace.path;
            if (!basePath) {
                basePath = "/";
            }
            components.shift();
        }
        else {
            // Relative to document
            let editorPath = editor.document.path;
            if (editorPath) {
                basePath = nova.path.dirname(editorPath);
            }
        }
        
        let lastComponent = components.pop();
        if (!lastComponent || lastComponent == "/") {
            lastComponent = "";
        }
        else {
            lastComponent = lastComponent.toLowerCase();
        }
        
        let directoryPath = basePath;
        if (components.length > 0) {
            directoryPath = nova.path.join(basePath, nova.path.join(...components));
        }
        
        let contents = null;
        try {
            contents = nova.fs.listdir(directoryPath);
        }
        catch (err) {
            return [];
        }
        
        let completions = [];
        
        for (let item of contents) {
            if (item.startsWith(".")) {
                // Ignore dot files
                continue;
            }
            
            let [name, ext] = nova.path.splitext(item);
            
            let stat = nova.fs.stat(nova.path.join(directoryPath, item));
            let isDirectory = stat.isDirectory();
            
            if (!isDirectory && !this.importableFileExtensions.includes(ext.toLowerCase())) {
                // Only include importable files and directories
                continue;
            }
            
            let lowercaseName = name.toLowerCase();
            
            if (lastComponent.length == 0 || lowercaseName.startsWith(lastComponent)) {
                let match = new CompletionItem(name, CompletionItemKind.File);
                
                let insertText = null;
                if (lastComponent.length > 0) {
                    insertText = nova.path.join(nova.path.dirname(substring), name);
                }
                else {
                    insertText = nova.path.join(substring, name);
                }
                
                if (isDirectory) {
                    insertText = insertText + "/";
                }
                
                match.insertText = insertText;
                match.filterText = insertText;
                match.range = new Range(context.position - substring.length, context.position);
                match.path = nova.path.join(directoryPath, item);
                completions.push(match);
            }
        }
        
        return completions;
    }
}

module.exports = new JavaScriptCompletionsAssistant();
